@@ -270,9 +270,10 @@ char	*getenv();
 #if WIN32
 
 #include <win32lib.h>
+#include <gs32lib.h>
 HANDLE hGsThread = NULL;
 unsigned char *gs_device = NULL;
-HANDLE hGsDataIn, hGsDataOut; /* Events to synchronize threads */
+HANDLE hGsDataIn = 0, hGsDataOut = 0; /* Events to synchronize threads */
 /* Arguments to gs dll */
 char *gs_argv[] = { "gswin32c.exe",
 		    "-dNOGC",
@@ -288,10 +289,6 @@ char *gs_argv[] = { "gswin32c.exe",
 		    NULL
   };
 char *buffer_stdin; /* This is the buffer from where data are taken. */
-#if 0
-#define	WIFSIGNALED(stat_val)	(((stat_val) & ((DWORD)0x0F0000000L)) != 0)
-#define	WEXITSTATUS(stat_val)	(((stat_val) >> 8) & 0x0FF)
-#endif
 
 #else /* not WIN32 */
 
@@ -536,13 +533,45 @@ exit_toto_too P1H(void)
 #if !WIN32
 	if (gs_pid != 0)
 	    kill(gs_pid, SIGKILL);
+#else
+	if (hGsThread) {
+	  switch (WaitForSingleObject(hGsThread, 2000)) {
+	  case WAIT_OBJECT_0:
+	    CloseHandle(hGsThread);
+	    hGsThread = NULL;
+	    break;
+	  case WAIT_TIMEOUT:
+	    fprintf(stderr, "Timeout waiting for Gs thread.\n");
+	    break;
+	  case WAIT_FAILED:
+	    fprintf(stderr, "WaitForSingleObject failed on Gs thread (Error code %d).\n",
+		    GetLastError());
+	    break;
+	  default:
+	    break;
+	  }
+
+	  if (hGsThread) {
+	    if (TerminateThread(hGsThread, 1) == 0) {
+	      fprintf(stderr, "... couldn't terminate gs thread\n");
+	    }
+	    CloseHandle(hGsThread);
+	    /* FIXME : is it right to call this ? */
+	    gs_dll_release();
+	  }	  
+	}
+
+	if (hGsDataIn)
+	  CloseHandle(hGsDataIn);
+	if (hGsDataOut)
+	  CloseHandle(hGsDataOut);
+
+#endif
 	if (pk_file != NULL) {
 	    fclose(pk_file);
 	    if (unlink(xfilename) != 0) perror("unlink");
 	}
-#else
-	gs_dll_release();
-#endif
+
 	_exit(1);
 }
 
@@ -703,11 +732,6 @@ handle_sigterm(DWORD dwCtrlType)
 	    case CTRL_C_EVENT:
 	    case CTRL_BREAK_EVENT:
 		fprintf(stderr, "...exiting\n");
-		if (hGsThread) {
-		  if (TerminateThread(hGsThread, 1) == 0) {
-		    fprintf(stderr, "... couldn't terminate gs thread\n");
-		  }
-		}
 		exit_toto_too();
 		return FALSE;
 	    default:
@@ -768,13 +792,6 @@ gsdll_callback(int message, char *str, unsigned long count)
 #endif
     data_out = buffer;
 
-    if (str == (char *)NULL || count == 0) {
-      data_eof = True;
-      data_end = data_out;
-      /* Tell data_fillbuf() */
-      SetEvent(hGsDataIn);
-      return 0;
-    }
     n = (count >= BUFSIZE ? BUFSIZE : count);
     memcpy(data_out, str, n);
     data_end = data_out + n;
@@ -811,11 +828,11 @@ gsdll_callback(int message, char *str, unsigned long count)
 
   case GSDLL_POLL:
 #if 0
-    fprintf(stderr, "GS: Poll sent!\n");
+    fprintf(stderr, "GS: Poll sent (%d)!\n", 0);
 #endif
-    return 0; /* no error */
+    return 0; /* no error ? */
   default:
-    fprintf(stdout,"Callback: Unknown message=%d\n",message);
+    fprintf(stdout,"%s: gs callback: unknown message=%d\n",progname, message);
     break;
   }
   return 0;
@@ -829,23 +846,35 @@ DWORD WINAPI Win32GsSendData(LPVOID lpParam)
 {
   int ret;
 
-  if (gs_locate() == NULL) {
-    fprintf(stderr, "Can't locate Ghostscript ! Exiting ...\n");
-    return EXIT_FAILURE;
-  }
-
   gs_dll_initialize();
 
-  if ((ret = (*pgsdll_init)(gsdll_callback,
-			    NULL,
-			    sizeof(gs_argv) / sizeof(char *) - 1,
-			    gs_argv)) == 0) {
-    /* Should not happen : gs should quit right after being initialized. */
-    (*pgsdll_exit)();
-  }
+  ret = (*pgsdll_init)(gsdll_callback,
+		       NULL,
+		       sizeof(gs_argv) / sizeof(char *) - 1,
+		       gs_argv);
 
-  if (ret != 0 && ret != GSDLL_INIT_QUIT)
-      fprintf(stderr, "gsdll_init returned %d\n", ret);
+  switch (ret) {
+  case 0:
+    /* Should not happen : gs should quit
+       right after being initialized. */
+    (*pgsdll_exit)();
+    /* FIXME: this is working, but we could expect something cleaner ! */
+    /*  GenerateConsoleCtrlEvent(CTRL_BREAK_EVENT, 0); */
+    break;
+  case GSDLL_INIT_QUIT:
+    break;
+  default:
+    (*pgsdll_exit)();
+    break;
+  }
+#if DEBUG  
+  fprintf(stderr, "%s: gsdll_init returned %d\n", progname, ret);
+#endif
+  WaitForSingleObject(hGsDataOut, INFINITE);
+  data_eof = True;
+  data_end = data_out;
+  /* Tell data_fillbuf() */
+  SetEvent(hGsDataIn);
 
   gs_dll_release();
 
@@ -914,7 +943,10 @@ wait_for_gs P1H(void)
 static void
 data_fillbuf P1H(void)
 {
-  /* wait until data are available. First tell gs he can fill the buffer in. */
+  if (data_eof)
+    return;
+
+  /* wait until data are available. First tell gs it can fill the buffer in. */
   SetEvent(hGsDataOut);
   /* wait for the data. */
   switch (WaitForSingleObject(hGsDataIn, INFINITE)) {
@@ -2054,7 +2086,6 @@ main P2C(int, argc, char **, argv)
 #endif
 #if WIN32
 	DWORD idGsThread;
-	SECURITY_ATTRIBUTES sa = { sizeof(SECURITY_ATTRIBUTES), NULL, TRUE };
 #else /* not WIN32 */
 #if !_AMIGA
 	int		std_out[2];
@@ -2372,6 +2403,10 @@ Author of gsftopk: Paul Vojta.");
 		*p = '\0';
 #ifdef KPATHSEA
 		searchpath = kpse_find_file(q, kpse_tex_ps_header_format,true);
+#ifdef WIN32
+		/* Be safe for Ghostscript's sake */
+		dostounix_filename(searchpath);
+#endif /* WIN32 */
 		f = searchpath ? fopen(searchpath, FOPEN_R_MODE) : NULL;
 #else
 		f = search(config_file_header_path, "DVIPSHEADERS", q);
@@ -2621,10 +2656,15 @@ Author of gsftopk: Paul Vojta.");
 
 #else /* WIN32 */
 
+	if (gs_locate() == NULL) {
+	  fprintf(stderr, "\nCan't locate Ghostscript ! Exiting ...\n");
+	  return EXIT_FAILURE;
+	}
+
 	SetConsoleCtrlHandler(handle_sigterm, TRUE);
 
-	hGsDataIn = CreateEvent(&sa, FALSE, FALSE, "gsDataIn");
-	hGsDataOut = CreateEvent(&sa, FALSE, FALSE, "gsDataOut");
+	hGsDataIn = CreateEvent(NULL, FALSE, FALSE, "gsDataIn");
+	hGsDataOut = CreateEvent(NULL, FALSE, FALSE, "gsDataOut");
 
 	gs_argv[3] = substarg;
 	gs_argv[6] = searchpath;
@@ -2635,12 +2675,12 @@ Author of gsftopk: Paul Vojta.");
 
 	buffer_stdin = concat(designstr, charlist);
 
-	if ((hGsThread = CreateThread(&sa, /* security attributes */
-				      0,	/* default stack size */
+	if ((hGsThread = CreateThread(NULL,            /* security attributes */
+				      0,               /* default stack size */
 				      Win32GsSendData, /* start address of thread */
-				      0,	/* parameter */
-				      0,	/* creation flags */
-				      &idGsThread /* thread id */
+				      0,               /* parameter */
+				      0,               /* creation flags */
+				      &idGsThread      /* thread id */
 				      )) == NULL)
 	  Win32Error("CreateThread");
 
@@ -2722,20 +2762,42 @@ Author of gsftopk: Paul Vojta.");
 
 #elif WIN32
 
-	switch (WaitForSingleObject(hGsThread, INFINITE)) {
-	case WAIT_OBJECT_0:
-	  break;
-	case WAIT_TIMEOUT:
-	  fprintf(stderr, "Gs thread did not terminate.\n");
-	  break;
-	case WAIT_FAILED:
-	  break;
-	default:
-	  fprintf(stderr, "WaitForSingleObject failed on Gs thread for unknown reason.\n");
+	/* Release data buffer, enable thread to terminate */
+	SetEvent(hGsDataOut);
+
+#ifdef DEBUG
+	fprintf(stderr, "Waiting for thread ... \n");
+#endif
+	if (hGsThread) {
+	  switch (WaitForSingleObject(hGsThread, 2000)) {
+	  case WAIT_OBJECT_0:
+	    CloseHandle(hGsThread);
+	    hGsThread = NULL;
+	    break;
+	  case WAIT_TIMEOUT:
+	    fprintf(stderr, "Timeout waiting for Gs thread.\n");
+	    break;
+	  case WAIT_FAILED:
+	    fprintf(stderr, "WaitForSingleObject failed on Gs thread (Error code %d).\n",
+		    GetLastError());
+	    break;
+	  default:
+	    break;
+	  }
 	}
-	CloseHandle(hGsThread);
-	CloseHandle(hGsDataIn);
-	CloseHandle(hGsDataOut);
+
+	if (hGsThread) {
+	  if (TerminateThread(hGsThread, 1) == 0) {
+	    fprintf(stderr, "... couldn't terminate gs thread\n");
+	  }
+	  CloseHandle(hGsThread);
+	  /* FIXME : is it right to call this ? */
+	  gs_dll_release();
+	}
+	if (hGsDataIn)
+	  CloseHandle(hGsDataIn);
+	if (hGsDataOut)
+	  CloseHandle(hGsDataOut);
 
 #else /* neither _AMIGA nor WIN32 */
 
