@@ -273,7 +273,6 @@ int argc; char *argv[];
       	    break;
       	 case 'd':
       	    debug = 1;
-	    pstat = my_stat;
 	    break;
       	 case 'e':
       	    if (*++argv[0] == '\0') {
@@ -337,7 +336,6 @@ int argc; char *argv[];
       	    break;
       	 case 'v':
       	    verbose = 1;
-	    pfopen = my_fopen; /* show all file opens */
 	    break;
       	 default:
       	    fatal("%s: %c illegal option\n", myname, c);
@@ -697,16 +695,16 @@ void Xfree(void *p)
 void FontDefaultFormat(int *bit, int *byte, 
 		       CharInfoRec *glyphs, int *scan) { ; }
 void FontFileRegisterRenderer(FontRendererRec *foo) { ; }
-FontParseXLFDName(char *scaledName, FontScalablePtr Vals, int foo ) { ; }
+void FontParseXLFDName(char *scaledName, FontScalablePtr Vals, int foo ) { ; }
 FontComputeInfoAccelerators(FontInfoPtr foo) { ; }
 #else
-FontDefaultFormat() { ; }
+void FontDefaultFormat() { ; }
  
-FontFileRegisterRenderer() { ; }
+void FontFileRegisterRenderer() { ; }
  
 GenericGetBitmaps() { ; }
 GenericGetExtents() { ; }
  
-FontParseXLFDName() { ; }
+void FontParseXLFDName() { ; }
 FontComputeInfoAccelerators() { ; }
 #endif
