@@ -106,6 +106,12 @@ static string user_progname;
 /* The C version of what might wind up in DUMP_VAR.  */
 static const_string dump_name;
 
+/* The C version of the jobname, if given. */
+static const_string job_name;
+
+/* Full source file name. */
+extern string fullnameoffile;
+
 /* The filename for dynamic character translation, or NULL.  */
 string translate_filename;
 string default_translate_filename;
@@ -119,7 +125,9 @@ static void parse_src_specials_option P1H(const_string);
 /* The main body of the WEB is transformed into this procedure.  */
 extern TEXDLL void mainbody P1H(void);
 
-static void maybe_parse_first_line P1H(void);
+/* Parsing a first %&-line in the input file. */
+static void parse_first_line P1H(void);
+
 static void parse_options P2H(int, string *);
 
 #ifdef MP
@@ -166,10 +174,17 @@ maininit P2C(int, ac, string *, av)
   kpse_set_program_name (argv[0], user_progname);
 
   /* If no dump default yet, and we're not doing anything special on
-     this run, look at the first line of the main input file for a
-     %&<dumpname> specifier.  */
-  if (!dump_name || !translate_filename) {
-    maybe_parse_first_line ();
+     this run, we may want to look at the first line of the main input
+     file for a %&<dumpname> specifier.  */
+  if (!parsefirstlinep) {
+    string parse_first_line = kpse_var_value ("parse_first_line");
+    parsefirstlinep = (parse_first_line
+                       && (*parse_first_line == 't'
+                           || *parse_first_line == 'y'
+                           || *parse_first_line == '1'));
+  }
+  if (parsefirstlinep && (!dump_name || !translate_filename)) {
+    parse_first_line ();
   }
   /* Check whether there still is no translate_filename known.  If so,
      use the default_translate_filename. */
@@ -223,7 +238,13 @@ maininit P2C(int, ac, string *, av)
 #ifdef MF
   kpse_set_program_enabled (kpse_mf_format, MAKE_TEX_MF_BY_DEFAULT,
                             kpse_src_compile);
+  kpse_set_program_enabled (kpse_base_format, MAKE_TEX_FMT_BY_DEFAULT,
+                            kpse_src_compile);
 #endif /* MF */
+#ifdef MP
+  kpse_set_program_enabled (kpse_mem_format, MAKE_TEX_FMT_BY_DEFAULT,
+                            kpse_src_compile);
+#endif /* MP */
 #ifdef TeX
 #ifdef Omega
   kpse_set_program_enabled (kpse_ocp_format, MAKE_OMEGA_OCP_BY_DEFAULT,
@@ -231,14 +252,15 @@ maininit P2C(int, ac, string *, av)
   kpse_set_program_enabled (kpse_ofm_format, MAKE_OMEGA_OFM_BY_DEFAULT,
                             kpse_src_compile);
   kpse_set_program_enabled (kpse_tfm_format, false, kpse_src_compile);
-  kpse_set_program_enabled (kpse_tex_format, MAKE_TEX_TEX_BY_DEFAULT,
-                            kpse_src_compile);
 #else
   kpse_set_program_enabled (kpse_tfm_format, MAKE_TEX_TFM_BY_DEFAULT,
                             kpse_src_compile);
+#endif /* !Omega */
   kpse_set_program_enabled (kpse_tex_format, MAKE_TEX_TEX_BY_DEFAULT,
                             kpse_src_compile);
-#endif /* !Omega */
+  kpse_set_program_enabled (kpse_fmt_format, MAKE_TEX_FMT_BY_DEFAULT,
+                            kpse_src_compile);
+
   if (!shellenabledp) {
     string shell_escape = kpse_var_value ("shell_escape");
     shellenabledp = (shell_escape
@@ -249,15 +271,6 @@ maininit P2C(int, ac, string *, av)
   if (!outputcomment) {
     outputcomment = kpse_var_value ("output_comment");
   }
-  if (!srcspecialsoption) {
-       /* Look in texmf.cnf for a value */
-       string src_opt = kpse_var_value("src_specials");
-       /* No value means no specials. */
-       if (src_opt != NULL) {
-         last_source_name = xstrdup("");
-         parse_src_specials_option(src_opt);
-       }
-  }
 #endif /* TeX */
 }
 
@@ -472,8 +485,8 @@ ipc_snd P3C(int, n,  int, is_eof,  char *, data)
 
 /* This routine notifies the server if there is an eof, or the filename
    if a new DVI file is starting.  This is the routine called by TeX.
-   Omega defines str_start(#) as str_start_ar(# - biggest_char), with
-   biggest_char = 65535 ([4.38], [1.12] om16bit.c).  */
+   Omega defines str_start(#) as str_start_ar[# - too_big_char], with
+   too_big_char = biggest_char + 1 = 65536 (omstr.ch).*/
 
 void
 ipcpage P1C(int, is_eof)
@@ -491,15 +504,15 @@ ipcpage P1C(int, is_eof)
 #ifndef Omega
     len = strstart[outputfilename + 1] - strstart[outputfilename];
 #else
-    len = strstartar[outputfilename + 1 - 65535L] -
-            strstartar[outputfilename - 65535L];
+    len = strstartar[outputfilename + 1 - 65536L] -
+            strstartar[outputfilename - 65536L];
 #endif
     name = (string)xmalloc (len + 1);
 #ifndef Omega
     strncpy (name, &strpool[strstart[outputfilename]], len);
 #else
     for (i=0; i<len; i++)
-      name[i] =  strpool[i+strstartar[outputfilename - 65535L]];
+      name[i] =  strpool[i+strstartar[outputfilename - 65536L]];
 #endif
     name[len] = 0;
     
@@ -640,7 +653,13 @@ setupcharset P1H(void)
   /* The expansion is nonempty if the variable is set.  */
   if (translate_filename) {
     read_char_translation_file ();
-  } else {
+  }
+#if 0
+  /* Code to set the isprintable array based on the locale.  The code
+   * was removed because in this form it affected not just output to
+   * the terminal and log file, but also the results of \write.
+   */
+  else {
     /* Use the locale to adjust the isprintable array. */
     for (c = 0; c <= 255; c++) {
       if (!isprintable[c] && isprint(c)) {
@@ -648,6 +667,7 @@ setupcharset P1H(void)
       }
     }
   }
+#endif
 }  
 #endif /* !Omega */
 #endif /* TeX || MF || MP [character translation] */
@@ -684,6 +704,8 @@ static struct option long_options[]
 #endif /* TeX */
 #if defined (TeX) || defined (MF) || defined (MP)
       { "file-line-error-style",  0, &filelineerrorstylep, 1 },
+      { "jobname",                1, 0, 0 },
+      { "parse-first-line",       0, &parsefirstlinep, 1 },
 #ifndef Omega
       { "translate-file",         1, 0, 0 },
       { "default-translate-file", 1, 0, 0 },
@@ -728,6 +750,9 @@ parse_options P2C(int, argc,  string *, argv)
     } else if (ARGUMENT_IS ("progname")) {
       user_progname = optarg;
 
+    } else if (ARGUMENT_IS ("jobname")) {
+      job_name = optarg;
+      
     } else if (ARGUMENT_IS (DUMP_OPTION)) {
       dump_name = optarg;
       if (!user_progname) user_progname = optarg;
@@ -763,10 +788,11 @@ parse_options P2C(int, argc,  string *, argv)
     } else if (ARGUMENT_IS ("src-specials")) {
        last_source_name = xstrdup("");
        /* Option `--src" without any value means `auto' mode. */
-       if (optarg == 0) {
+       if (optarg == NULL) {
          insertsrcspecialeverypar = true;
          insertsrcspecialauto = true;
          srcspecialsoption = true;
+         srcspecialsp = true;
        } else {
           parse_src_specials_option(optarg);
        }
@@ -865,6 +891,10 @@ parse_src_specials_option P1C(const_string, opt_list)
     tok = strtok(0, ", ");
   }
   free(toklist);
+  srcspecialsp=insertsrcspecialauto | insertsrcspecialeverypar |
+    insertsrcspecialeveryparend | insertsrcspecialeverycr |
+    insertsrcspecialeverymath |  insertsrcspecialeveryhbox |
+    insertsrcspecialeveryvbox | insertsrcspecialeverydisplay;
   srcspecialsoption = true;
 }
 #endif
@@ -875,22 +905,12 @@ parse_src_specials_option P1C(const_string, opt_list)
    FORMAT is a readable dump file, then set DUMP_VAR to FORMAT.
    Also call kpse_reset_program_name to ensure the correct paths for the
    format are used.  */
-
 static void
-maybe_parse_first_line P1H(void)
+parse_first_line P1H(void)
 {
   char first_char = optind < argc ? argv[optind][0] : 0;
-  string parse_first_line = kpse_var_value ("parse_first_line");
 
-  /* Default to parsing the first line. */
-  if (!parse_first_line) parse_first_line = "t";
-
-  if ((*parse_first_line == 't'
-       || *parse_first_line == 'y'
-       || *parse_first_line == '1')
-      && first_char
-      && first_char != '&'
-      && first_char != '\\') {
+  if (first_char && first_char != '&' && first_char != '\\') {
     /* If the file can't be found, don't look too hard now.  We'll
        detect that it's missing in the normal course of things and give
        the error then.  */
@@ -953,6 +973,7 @@ maybe_parse_first_line P1H(void)
             s = *parse+16;
           }
           /* Just set the name, no sanity checks here. */
+          /* FIXME: remove trailing spaces. */
           if (s && *s) {
             translate_filename = xstrdup(s);
           }
@@ -1472,12 +1493,20 @@ maketexstring(const_string s)
 }
 #endif
 
-int
+strnumber
 makefullnamestring()
 {
-  extern string fullnameoffile;
   return maketexstring(fullnameoffile);
 }
+
+strnumber
+getjobname()
+{
+    strnumber ret = curname;
+    if (job_name != NULL)
+      ret = maketexstring(job_name);
+    return ret;
+}
 #endif
 
 #if defined(TeX)
@@ -1501,19 +1530,19 @@ compare_paths P2C(const_string, p1, const_string, p2)
 string
 gettexstring P1C(strnumber, s)
 {
-  size_t i, len;
+  poolpointer i, len;
   string name;
 #ifndef Omega
   len = strstart[s + 1] - strstart[s];
 #else
-  len = strstartar[s + 1 - 65535L] - strstartar[s - 65535L];
+  len = strstartar[s + 1 - 65536L] - strstartar[s - 65536L];
 #endif
   name = (string)xmalloc (len + 1);
 #ifndef Omega
   strncpy (name, (string)&strpool[strstart[s]], len);
 #else
   /* Don't use strncpy.  The strpool is not made up of chars. */
-  for (i=0; i<len; i++) name[i] =  strpool[i+strstartar[s - 65535L]];
+  for (i=0; i<len; i++) name[i] =  strpool[i+strstartar[s - 65536L]];
 #endif
   name[len] = 0;
   return name;
@@ -1522,7 +1551,6 @@ gettexstring P1C(strnumber, s)
 boolean
 isnewsource P2C(strnumber, srcfilename, int, lineno)
 {
-  size_t len;
   char *name = gettexstring(srcfilename);
   return (compare_paths(name, last_source_name) != 0 || lineno != last_lineno);
 }
@@ -1531,8 +1559,6 @@ void
 remembersourceinfo P2C(strnumber, srcfilename,
                                           int, lineno)
 {
-  size_t len;
-
   if (last_source_name)
        free(last_source_name);
   last_source_name = gettexstring(srcfilename);
