@@ -1,9 +1,32 @@
+/*
+Copyright 1996,1997,1998,1999,2000,2001,2002,2003 Han The Thanh, <thanh@pdftex.org>
+
+This file is part of pdfTeX.
+
+pdfTeX is free software; you can redistribute it and/or modify
+it under the terms of the GNU General Public License as published by
+the Free Software Foundation; either version 2 of the License, or
+(at your option) any later version.
+
+pdfTeX is distributed in the hope that it will be useful,
+but WITHOUT ANY WARRANTY; without even the implied warranty of
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+GNU General Public License for more details.
+
+You should have received a copy of the GNU General Public License
+along with pdfTeX; if not, write to the Free Software
+Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+$Id: //depot/Build/source.development/TeX/texk/web2c/pdftexdir/pdftoepdf.cc#35 $
+*/
+
 #include <stdlib.h>
 #include <math.h>
 #include <stddef.h>
 #include <stdio.h>
 #include <string.h>
 #include <ctype.h>
+#include <aconf.h>
 #include <GString.h>
 #include <gmem.h>
 #include <gfile.h>
@@ -17,19 +40,45 @@
 #include "Page.h"
 #include "GfxFont.h"
 #include "PDFDoc.h"
-#include "Params.h"
+#include "GlobalParams.h"
 #include "Error.h"
 
 #include "epdf.h"
 
+static const char perforce_id[] = 
+    "$Id: //depot/Build/source.development/TeX/texk/web2c/pdftexdir/pdftoepdf.cc#35 $";
+
+/* we avoid reading all necessary kpathsea headers, but we need xstrdup */
+#ifdef __cplusplus
+extern "C" {
+  extern char *xstrdup (char *);
+}
+#else
+  extern char *xstrdup (char *);
+#endif
+
+#ifdef __cplusplus
+extern "C" {
 // FIXME: This definition is a duplicate from ../pdftexcoerce.h, which is
 // not usable here because of different macro definitions.
 extern integer pdflastpdfboxspec ;
+}
+#endif
 
-// if pdfTeX should care about a /CropBox or should only use the /MediaBox.
-// FIXME: This is needed by ArtCom because we currently cannot specify the
-// usage of the /MediaBox in LaTeX with pdftex.def
-#define USE_CROPBOX 0
+/*
+ * Used flags below:
+ * PDFTEX_COPY_PAGEGROUP
+ *      if pdfTeX should copy a page group (a new feature in pdf1.4 for
+ *      transparency) of an included file. The current support for this is
+ *      most likely broken. pdfTeX will at least give a warning if this flag
+ *      is not set. Surprisingly Acrobat and Jaws display files without a
+ *      page group correctly, so it might be safe to not set the flag.
+ *  See also PDFTEX_COPY_PAGEGROUP_NOFAIL
+ * PDFTEX_COPY_PAGEGROUP_NOFAIL
+ *      if set to false pdfTeX will treat a page group in an included file as
+ *      an error and abort gracefully. This is only evaluated if
+ *      PDFTEX_COPY_PAGEGROUP is set.
+ */
 
 // FIXME: These definitions are duplicates from pdftexd.h
 /* #define PDF_PDF_BOX_SPEC_MEDIA 0 */
@@ -107,11 +156,15 @@ struct PdfDocument {
     PDFDoc *doc;
     XRef *xref;
     InObj *inObjList;
+    int occurences;     // number of references to the document; the doc can be
+                        // deleted when this is negative
     PdfDocument *next;
 };
 
 static PdfDocument *pdfDocuments = 0;
 
+static XRef *xref = 0;
+
 // Returns pointer to PdfDocument record for PDF file.
 // Creates a new record if it doesn't exist yet.
 // xref is made current for the document.
@@ -123,15 +176,24 @@ static PdfDocument *find_add_document(char *file_name)
         p = p->next;
     if (p) {
         xref = p->xref;
-	return p;
+        (p->occurences)++;
+#ifdef DEBUG
+        fprintf(stderr, "\nIncrementing %s (%d)\n", p->file_name, p->occurences);
+#endif
+        return p;
     }
     p = new PdfDocument;
-    p->file_name = strdup(file_name);
+    p->file_name = xstrdup(file_name);
     p->xref = xref = 0;
+    p->occurences = 0;
+#ifdef DEBUG
+    fprintf(stderr, "\nCreating %s (%d)\n", p->file_name, p->occurences);
+#endif
     GString *docName = new GString(p->file_name);
     p->doc = new PDFDoc(docName);  // takes ownership of docName
-    if (!p->doc->isOk() || !p->doc->okToPrint())
+    if (!p->doc->isOk() || !p->doc->okToPrint()) {
         pdftex_fail("xpdf: reading PDF image failed");
+        }
     p->inObjList = 0;
     p->next = pdfDocuments;
     pdfDocuments = p;
@@ -161,6 +223,20 @@ static void delete_document(PdfDocument *pdf_doc)
     delete pdf_doc;
 }
 
+// Replacement for 
+//      Object *initDict(Dict *dict1){ initObj(objDict); dict = dict1; return this; }
+static void initDictFromDict (PdfObject &obj, Dict *dict)
+{
+    obj->initDict(xref);
+    for (int i = 0,
+         l = dict->getLength(); 
+         i < l; 
+         i++) {
+        Object obj1;
+        obj->dictAdd(copyString(dict->getKey(i)), dict->getValNF(i, &obj1));
+        }
+}    
+
 // --------------------------------------------------------------------
 
 static int addEncoding(GfxFont *gfont)
@@ -204,9 +280,9 @@ static int addInObj(InObjType type, Ref ref, int f, integer e)
             }
             q = p;
         }
-	// it is important to add new objects at the end of the list,
-	// because new objects are being added while the list is being
-	// written out.
+    // it is important to add new objects at the end of the list,
+    // because new objects are being added while the list is being
+    // written out.
         q->next = n;
     }
     n->num = pdfnewobjnum();
@@ -220,7 +296,7 @@ static void copyName(char *s)
     pdf_puts("/");
     for (; *s != 0; s++) {
         if (isdigit(*s) || isupper(*s) || islower(*s) || *s == '_' ||
-	    *s == '.' || *s == '-' )
+        *s == '.' || *s == '-' )
             pdfout(*s);
         else
             pdf_printf("#%.2X", *s & 0xFF);
@@ -288,10 +364,8 @@ static void copyStream(Stream *str)
 {
     int c;
     str->reset();
-    if (pdfcrypting) pdfcrypt_initkey();
     while ((c = str->getChar()) != EOF)
-        if (pdfcrypting) pdfout(pdfcrypt_byte(c));
-        else             pdfout(c);
+        pdfout(c);
 }
 
 static void copyProcSet(Object *obj)
@@ -303,7 +377,7 @@ static void copyProcSet(Object *obj)
                     obj->getTypeName());
     pdf_puts("/ProcSet [ ");
     for (i = 0, l = obj->arrayGetLength(); i < l; ++i) {
-        obj->arrayGet(i, &procset);
+        obj->arrayGetNF(i, &procset);
         if (!procset->isName())
             pdftex_fail("pdf inclusion: invalid ProcSet entry type <%s>", 
                         procset->getTypeName());
@@ -323,12 +397,12 @@ static void copyFont(char *tag, Object *fontRef)
     Ref ref = fontRef->getRef();
     for (p = inObjList; p; p = p->next) {
         if (p->ref.num == ref.num && p->ref.gen == ref.gen) {
-	  copyName(tag);
-	  pdf_printf(" %d 0 R ", p->num);
-	  return;
-	}
+      copyName(tag);
+      pdf_printf(" %d 0 R ", p->num);
+      return;
     }
-    fontRef->fetch(&fontdict);
+    }
+    fontRef->fetch(xref, &fontdict);
     if (!fontdict->isDict())
         pdftex_fail("pdf inclusion: invalid font dict type <%s>", 
                     fontdict->getTypeName());
@@ -339,9 +413,9 @@ static void copyFont(char *tag, Object *fontRef)
     /* only handle Type1 fonts; others will be copied */
     if (strcmp(subtype->getName(), "Type1") != 0 ) {
         copyName(tag);
-	pdf_puts(" ");
-	copyObject(fontRef);
-	return;
+    pdf_puts(" ");
+    copyObject(fontRef);
+    return;
     }
     fontdict->dictLookup("BaseFont", &basefont);
     if (!basefont->isName())
@@ -350,18 +424,18 @@ static void copyFont(char *tag, Object *fontRef)
     fontmap = lookup_fontmap(basefont->getName());
     if (fontmap >= 0 && is_type1(fontmap) &&
         fontdict->dictLookupNF("FontDescriptor", &fontdescRef) && 
-        fontdescRef->isRef() && fontdescRef->fetch(&fontdesc) &&
-	fontdesc->isDict()) {
+        fontdescRef->isRef() && fontdescRef->fetch(xref, &fontdesc) &&
+    fontdesc->isDict()) {
         if (fontdesc->dictLookup("CharSet", &charset) && 
             charset->isString() && is_subsetable(fontmap))
-	    mark_glyphs(fontmap, charset->getString()->getCString());
+        mark_glyphs(fontmap, charset->getString()->getCString());
         else
             embed_whole_font(fontmap);
         addFontDesc(fontdescRef->getRef(), fontmap);
     }
     copyName(tag);
     if (fontdesc->isDict()) {
-        gfont = new GfxFont(tag, fontRef->getRef(), fontdict->getDict());
+        gfont = GfxFont::makeFont(xref, tag, fontRef->getRef(), fontdict->getDict());
         pdf_printf(" %d 0 R ", addFont(fontRef->getRef(), fontmap, 
                                        addEncoding(gfont)));
     }
@@ -390,51 +464,47 @@ static void copyFontResources(Object *obj)
 
 static void copyOtherResources(Object *obj, char *key)
 {
-    PdfObject obj1;
-    int i, l;
+    /* copies all other resources (write_epdf handles Fonts and ProcSets),
+     * but gives a warning if an object is not a dictionary.
+     */
     if (!obj->isDict())
-        pdftex_fail("pdf inclusion: invalid other resources dict type <%s>", 
+        //FIXME: Write the message only to the log file
+        pdftex_warn("pdf inclusion: invalid other resources dict type <%s>, "
+                    "copying it anyway.", 
                     obj->getTypeName());
     copyName(key);
-    pdf_puts(" << ");
-    for (i = 0, l = obj->dictGetLength(); i < l; ++i) {
-        obj->dictGetValNF(i, &obj1);
-	copyName(obj->dictGetKey(i));
-        if (obj1->isRef())
-            pdf_printf(" %d 0 R ", addOther(obj1->getRef()));
-        else {
-            pdf_puts(" ");
-            copyObject(&obj1);
-        }
-    }
-    pdf_puts(">>\n");
+    pdf_puts(" ");
+    copyObject(obj);
 }
 
-static char *convertNumToPDF(double n) {
-    /* converts double to string; very small and very large numbers are NOT
-     * converted to scientific notation.
-     * n must be a number or real confirming to the implementation limits of
-     * PDF as specified in appendix C.1 of the pdf ref.  
-     * These are:
-     * maximum value of ints is +2^32
-     * maximum value of reals is +2^15
-     * smalles values of reals is 1/(2^16)
-     */
-    static int precision = 6;
-    static int fact = (int)1E6;   /* must be 10^precision */
-    static double epsilon = 1E-6; /* must be 10^-precision */
+/* converts double to string; very small and very large numbers are NOT
+ * converted to scientific notation.
+ * n must be a number or real confirming to the implementation limits of PDF as
+ * specified in appendix C.1 of the pdf ref.  
+ * These are:
+ * maximum value of ints is +2^32
+ * maximum value of reals is +2^15
+ * smalles values of reals is 1/(2^16)
+ */
+static char *
+convertNumToPDF(double n) 
+{
+    static const int precision = 6;
+    static const int fact = (int) 1E6;	/* must be 10^precision */
+    static const double epsilon = 0.5E-6; /* 2epsilon must be 10^-precision */
     static char buf[64];
     // handle very small values: return 0
     if (fabs(n) < epsilon) {buf[0] = '0'; buf[1] = '\0';}
     else {
         char ints[64];
         int bindex = 0, sindex = 0;
-        int ival, fval, i; 
+        int ival, fval; 
         // handle the sign part if n is negative
         if (n < 0) {
             buf[bindex++] = '-';
             n = -n;    
         }
+        n += epsilon; // for rounding
         // handle the integer part, simply with sprintf
         ival = (int)floor(n);
         n -= ival;
@@ -445,7 +515,8 @@ static char *convertNumToPDF(double n) {
         if (fval) {
             // set a dot
             buf[bindex++] = '.';
-            sindex = bindex + precision - 1;
+            sindex = bindex + precision;
+            buf[sindex--] = '\0';
             // fill up trailing zeros with the string terminator NULL
             while (((fval % 10) == 0) && (sindex >= bindex)) {
                 buf[sindex--] = '\0';
@@ -464,6 +535,7 @@ static char *convertNumToPDF(double n) {
 static void copyObject(Object *obj)
 {
     PdfObject obj1;
+    Dict *dict;
     int  i, l, c;
     Ref ref;
     char *p;
@@ -528,8 +600,8 @@ static void copyObject(Object *obj)
         pdf_puts(">>");
     }
     else if (obj->isStream()) {
-        obj1->initDict(obj->getStream()->getDict());
-        obj->getStream()->getDict()->incRef();
+        initDictFromDict (obj1, obj->streamGetDict());
+        obj->streamGetDict()->incRef();
         pdf_puts("<<\n");
         copyDict(&obj1);
         pdf_puts(">>\n");
@@ -554,20 +626,21 @@ static void copyObject(Object *obj)
 
 static void writeRefs()
 {
-    PdfObject obj1;
     InObj *r;
     for (r = inObjList; r != 0; r = r->next) {
         if (!r->written) {
-	    r->written = 1;
-	    zpdfbeginobj(r->num);
-	    xref->fetch(r->ref.num, r->ref.gen, &obj1);
-	    if (r->type == objFont || r->type == objFontDesc)
-	        copyFontDict(&obj1, r);
-	    else
+            Object obj1;
+        r->written = 1;
+        zpdfbeginobj(r->num);
+        xref->fetch(r->ref.num, r->ref.gen, &obj1);
+        if (r->type == objFont || r->type == objFontDesc)
+            copyFontDict(&obj1, r);
+        else
                 copyObject(&obj1);
-	    pdf_puts("\n");
-	    pdfendobj();
-	}
+        pdf_puts("\n");
+        pdfendobj();
+        obj1.free();
+    }
     }
 }
 
@@ -577,11 +650,15 @@ static void writeEncodings()
     char *glyphNames[MAX_CHAR_CODE + 1], *s;
     int i;
     for (r = encodingList; r != 0; r = r->next) {
-        for (i = 0; i <= MAX_CHAR_CODE; i++)
-            if ((s = r->font->getCharName(i)) != 0)
-                glyphNames[i] = s;
-            else
-                glyphNames[i] = notdef;
+      for (i = 0; i <= MAX_CHAR_CODE; i++) {
+      if (r->font->isCIDFont()) {
+        pdftex_warn("pdf inclusion: CID font included, encoding maybe wrong");
+      }
+      if ((s = ((Gfx8BitFont *)r->font)->getCharName(i)) != 0)
+        glyphNames[i] = s;
+      else
+        glyphNames[i] = notdef;
+      }
         write_enc(glyphNames, r->encoding);
     }
     for (r = encodingList; r != 0; r = n) {
@@ -591,13 +668,21 @@ static void writeEncodings()
     }
 }
 
-// Returns the page number.
-integer read_pdf_info(char *image_name, char *page_name, integer page_num)
+/* Reads various information about the pdf and sets it up for later inclusion.
+ * This will fail if the pdf version of the pdf is higher than
+ * minor_pdf_version_wanted or page_name is given and can not be found.
+ * It makes no sense to give page_name _and_ page_num.
+ * Returns the page number.
+ */
+integer 
+read_pdf_info(char *image_name, char *page_name, integer page_num,
+              integer minor_pdf_version_wanted, integer always_use_pdf_pagebox)
 {
     PdfDocument *pdf_doc;
     Page *page;
     int rotate;
-    Rectangle pagebox;
+    PDFRectangle *pagebox;
+    float pdf_version_found, pdf_version_wanted;
     // initialize
     if (!isInit) {
         // We should better not call xpdf's initParams, which would
@@ -606,66 +691,98 @@ integer read_pdf_info(char *image_name, char *page_name, integer page_num)
         // .xpdfrc, and we don't want the output PDF to depend on its
         // contents.  The following four lines replace the call to
         // initParams().
-        fontPath = (char **) gmalloc(sizeof(char *));
-	fontPath[0] = 0;
-	devFontMap = (DevFontMapEntry *) gmalloc(sizeof(DevFontMapEntry));
-	devFontMap[0].pdfFont = 0;
-	// initParams(xpdfConfigFile);
-        errorInit();
+
+    // read config file
+    globalParams = new GlobalParams("");
+        globalParams->setErrQuiet(gFalse);
         isInit = gTrue;
     }
     // open PDF file
     pdf_doc = find_add_document(image_name);
     epdf_doc = (void *) pdf_doc;
+#ifdef DEBUG
+    fprintf(stderr, "\nReading information on %s\n", pdf_doc->file_name);
+#endif
+    // check pdf version
+    // this works only for pdf 1.x -- but since any versions of pdf newer
+    // than 1.x will not be backwards compatible to pdf 1.x, pdfTeX will
+    // then have to changed drastically anyway.
+    pdf_version_found = pdf_doc->doc->getPDFVersion();
+    pdf_version_wanted = 1 + (minor_pdf_version_wanted * 0.1);
+    if (pdf_version_found > pdf_version_wanted) {
+        pdftex_fail("pdf inclusion: found pdf version <%.1f>, but at most version <%.1f> allowed",
+             pdf_version_found, pdf_version_wanted);
+        }
     epdf_num_pages = pdf_doc->doc->getCatalog()->getNumPages();
     if (page_name) {
         // get page by name
         GString name(page_name);
-	LinkDest *link = pdf_doc->doc->findDest(&name);
-	if (link == 0 || !link->isOk())
-	    pdftex_fail("pdf inclusion: invalid destination <%s>",
-			page_name);
-	Ref ref = link->getPageRef();
-	page_num =  pdf_doc->doc->getCatalog()->findPage(ref.num, ref.gen);
-	if (page_num == 0)
-	    pdftex_fail("pdf inclusion: destination is not a page <%s>",
-			page_name);
-	delete link;
+        LinkDest *link = pdf_doc->doc->findDest(&name);
+        if (link == 0 || !link->isOk())
+            pdftex_fail("pdf inclusion: invalid destination <%s>",
+                page_name);
+        Ref ref = link->getPageRef();
+        page_num = pdf_doc->doc->getCatalog()->findPage(ref.num, ref.gen);
+        if (page_num == 0)
+            pdftex_fail("pdf inclusion: destination is not a page <%s>",
+                page_name);
+        delete link;
     } else {
         // get page by number
         if (page_num <= 0 || page_num > epdf_num_pages)
-	    pdftex_fail("pdf inclusion: required page does not exist <%i>", 
-			epdf_num_pages);
+        pdftex_fail("pdf inclusion: required page does not exist <%i>", 
+            epdf_num_pages);
     }
     // get the required page
     page = pdf_doc->doc->getCatalog()->getPage(page_num);
 
-    switch (pdflastpdfboxspec) {
-#if USE_CROPBOX
-    case pdfpdfboxspeccrop:
-        pagebox = page->getLogicalCropBox();
-        break;
-    
-    case pdfpdfboxspecbleed:
-        pagebox = page->getLogicalBleedBox();
-        break;
-    
-    case pdfpdfboxspectrim:
-        pagebox = page->getLogicalTrimBox();
-        break;
-    
-    case pdfpdfboxspecart:
-        pagebox = page->getLogicalArtBox();
-        break;
-#endif
+    // get the pagebox (media, crop...) to use.
+    // always_use_pdf_pagebox can set in the config file to override the
+    // setting through pdfximage.
+    if (always_use_pdf_pagebox < 1) {
+        switch (pdflastpdfboxspec) {
+        case pdfpdfboxspeccrop:
+            pagebox = page->getCropBox();
+            break;
+        
+        case pdfpdfboxspecbleed:
+            pagebox = page->getBleedBox();
+            break;
+        
+        case pdfpdfboxspectrim:
+            pagebox = page->getTrimBox();
+            break;
+        
+        case pdfpdfboxspecart:
+            pagebox = page->getArtBox();
+            break;
 
-    default:
-        pagebox = page->getLogicalMediaBox();
-    }
-    epdf_width = pagebox.x2 - pagebox.x1;
-    epdf_height = pagebox.y2 - pagebox.y1;
-    epdf_orig_x = pagebox.x1;
-    epdf_orig_y = pagebox.y1;
+        default:
+            pagebox = page->getMediaBox();
+            }
+        }
+    else {
+        switch (always_use_pdf_pagebox) {
+        case 1 : 
+            pagebox = page->getMediaBox();
+            break;
+        case 2 : 
+            pagebox = page->getCropBox();
+            break;
+        case 3 : 
+            pagebox = page->getBleedBox();
+            break;
+        case 4 : 
+            pagebox = page->getTrimBox();
+            break;
+        default : // 5 and larger
+            pagebox = page->getArtBox();
+            }
+        }
+    epdf_width = pagebox->x2 - pagebox->x1;
+    epdf_height = pagebox->y2 - pagebox->y1;
+    epdf_orig_x = pagebox->x1;
+    epdf_orig_y = pagebox->y1;
     
     rotate = page->getRotate();
     // handle page rotation and adjust dimens as needed
@@ -682,15 +799,22 @@ integer read_pdf_info(char *image_name, char *page_name, integer page_num)
                 }
             }
         }
-    pdf_doc->xref = xref;
+    pdf_doc->xref = pdf_doc->doc->getXRef();
     return page_num;
 }
 
-void write_epdf(void)
+/* writes the current epf_doc.
+ * Here the included pdf is copied, so most errors that can happen during pdf
+ * inclusion will arise here.
+ */
+void 
+write_epdf(void)
 {
     Page *page;
     PdfObject contents, obj1, obj2;
+    PdfObject group, metadata, pieceinfo, separationInfo;
     Object info;
+    Dict *dict;
     char *key;
     int i, l;
     int rotate;
@@ -698,12 +822,16 @@ void write_epdf(void)
     char mybuf[2048];
     mybuf[0] = '\0';
     PdfDocument *pdf_doc = (PdfDocument *) epdf_doc;
+    (pdf_doc->occurences)--;
+#ifdef DEBUG
+    fprintf(stderr, "\nDecrementing %s (%d)\n", pdf_doc->file_name, pdf_doc->occurences);
+#endif
     xref = pdf_doc->xref;
     inObjList = pdf_doc->inObjList;
     encodingList = 0;
     page = pdf_doc->doc->getCatalog()->getPage(epdf_selected_page);
     rotate = page->getRotate();
-    Rectangle pagebox;
+    PDFRectangle *pagebox;
     // write the Page header
     pdf_puts("/Type /XObject\n");
     pdf_puts("/Subtype /Form\n");
@@ -719,29 +847,50 @@ void write_epdf(void)
         pdf_printf("/%s.InfoDict ", pdfkeyprefix);
         pdf_printf("%d 0 R \n", addOther(info.getRef()));
         }
-    
-    switch (epdf_page_box) {
-#if USE_CROPBOX
-    case pdfpdfboxspeccrop:
-        pagebox = page->getLogicalCropBox();
-        break;
   
-    case pdfpdfboxspecbleed:
-        pagebox = page->getLogicalBleedBox();
-        break;
-    
-    case pdfpdfboxspectrim:
-        pagebox = page->getLogicalTrimBox();
-        break;
-    
-    case pdfpdfboxspecart:
-        pagebox = page->getLogicalArtBox();
-        break;
-#endif
-   
-    default:
-        pagebox = page->getLogicalMediaBox();
-    }
+    // get the pagebox (media, crop...) to use.
+    // epdf_always_use_pdf_pagebox is a copy of always_use_pdf_pagebox which
+    // can set in the config file to override the setting through pdfximage.
+    if (epdf_always_use_pdf_pagebox < 1) {
+        switch (epdf_page_box) {
+        case pdfpdfboxspeccrop:
+            pagebox = page->getCropBox();
+            break;
+      
+        case pdfpdfboxspecbleed:
+            pagebox = page->getBleedBox();
+            break;
+        
+        case pdfpdfboxspectrim:
+            pagebox = page->getTrimBox();
+            break;
+        
+        case pdfpdfboxspecart:
+            pagebox = page->getArtBox();
+            break;
+       
+        default:
+            pagebox = page->getMediaBox();
+            }
+        }
+    else {
+        switch (epdf_always_use_pdf_pagebox) {
+        case 1 : 
+            pagebox = page->getMediaBox();
+            break;
+        case 2 : 
+            pagebox = page->getCropBox();
+            break;
+        case 3 : 
+            pagebox = page->getBleedBox();
+            break;
+        case 4 : 
+            pagebox = page->getTrimBox();
+            break;
+        default : // 5 and larger
+            pagebox = page->getArtBox();
+            }
+        }
 
     // handle page rotation
     if (rotate != 0) {
@@ -754,9 +903,9 @@ void write_epdf(void)
             // counterclockwise :-%
             tex_printf (", page is rotated %d degrees", rotate);
             switch (rotate) {
-                case  90: scale[1] = -1; scale[2] = 1; scale[5] = pagebox.x1 + pagebox.x2; break;
-                case 180: scale[0] = scale[3] = -1;    scale[4] = pagebox.x1 + pagebox.x2; scale[5] = pagebox.y1 + pagebox.y2; break; // width and height are exchanged
-                case 270: scale[1] = 1; scale[2] = -1; scale[4] = pagebox.y1 + pagebox.y2; break;
+                case  90: scale[1] = -1; scale[2] = 1; scale[5] = pagebox->x1 + pagebox->x2; break;
+                case 180: scale[0] = scale[3] = -1;    scale[4] = pagebox->x1 + pagebox->x2; scale[5] = pagebox->y1 + pagebox->y2; break; // width and height are exchanged
+                case 270: scale[1] = 1; scale[2] = -1; scale[4] = pagebox->y1 + pagebox->y2; break;
                 }
             }
         }
@@ -773,34 +922,93 @@ void write_epdf(void)
         scale[5]);
 
     pdf_printf("/BBox [%.5g %.5g %.5g %.5g]\n",
-               pagebox.x1,
-               pagebox.y1,
-               pagebox.x2,
-               pagebox.y2);
+               pagebox->x1,
+               pagebox->y1,
+               pagebox->x2,
+               pagebox->y2);
+
+    // write the page Group if it's there
+    if (page->getGroup() != NULL) {
+#if PDFTEX_COPY_PAGEGROUP
+#   if PDFTEX_COPY_PAGEGROUP_NOFAIL
+        // FIXME: This will most likely produce incorrect PDFs :-(
+        initDictFromDict(group, page->getGroup());
+        if (group->dictGetLength() > 0) {
+            pdf_puts("/Group ");
+            copyObject (&group);
+            pdf_puts("\n");
+        }
+#   else
+        // FIXME: currently we don't know how to handle Page Groups so we abort gracefully :-(
+        pdftex_fail("pdf inclusion: Page Group detected which pdfTeX can't handle. Sorry.");
+#   endif
+#else
+        // FIXME: currently we don't know how to handle Page Groups so we at least give a warning :-(
+        pdftex_warn("pdf inclusion: Page Group detected which pdfTeX can't handle. Ignoring it.");
+#endif
+    }
+    
+    // write the page Metadata if it's there
+    if (page->getMetadata() != NULL) {
+        metadata->initStream(page->getMetadata());
+        pdf_puts("/Metadata ");
+        copyObject (&metadata);
+        pdf_puts("\n");
+    }
+    
+    // write the page PieceInfo if it's there
+    if (page->getPieceInfo() != NULL) {
+        initDictFromDict (pieceinfo, page->getPieceInfo());
+        if (pieceinfo->dictGetLength() > 0) {
+            pdf_puts("/PieceInfo ");
+            copyObject (&pieceinfo);
+            pdf_puts("\n");
+        }
+    }
+    
+    // write the page SeparationInfo if it's there
+    if (page->getSeparationInfo() != NULL) {
+        initDictFromDict (separationInfo, page->getSeparationInfo());
+        if (separationInfo->dictGetLength() > 0) {
+            pdf_puts("/SeparationInfo ");
+            copyObject (&separationInfo);
+            pdf_puts("\n");
+        }
+    }
     
     // write the Resources dictionary
-    obj1->initDict(page->getResourceDict());
-    page->getResourceDict()->incRef();
-    if (!obj1->isDict())
-        pdftex_fail("pdf inclusion: invalid resources dict type <%s>", 
-                    obj1->getTypeName());
-    pdf_puts("/Resources <<\n");
-    for (i = 0, l = obj1->dictGetLength(); i < l; ++i) {
-        obj1->dictGetVal(i, &obj2);
-        key = obj1->dictGetKey(i);
-        if (strcmp("Font", key) == 0)
-            copyFontResources(&obj2);
-        else if (strcmp("ProcSet", key) == 0)
-            copyProcSet(&obj2);
-        else
-            copyOtherResources(&obj2, key);
+    if (page->getResourceDict() == NULL) {
+        // Resources can be missing (files without them have been spotted
+        // in the wild). This violates the pdf spec, which claims they are
+        // required, but all RIPs accept them.  
+        // We "replace" them with empty Resources.
+        pdftex_warn("pdf inclusion: no /Resources detected. Replacing with empty /Resources.");
+        pdf_puts("/Resources <<>>\n");
+        }
+    else {
+        initDictFromDict (obj1, page->getResourceDict());
+        page->getResourceDict()->incRef();
+        if (!obj1->isDict())
+            pdftex_fail("pdf inclusion: invalid resources dict type <%s>", 
+                        obj1->getTypeName());
+        pdf_puts("/Resources <<\n");
+        for (i = 0, l = obj1->dictGetLength(); i < l; ++i) {
+            obj1->dictGetVal(i, &obj2);
+            key = obj1->dictGetKey(i);
+            if (strcmp("Font", key) == 0)
+                copyFontResources(&obj2);
+            else if (strcmp("ProcSet", key) == 0)
+                copyProcSet(&obj2);
+            else
+                copyOtherResources(&obj2, key);
+            }
+        pdf_puts(">>\n");
     }
-    pdf_puts(">>\n");
     // write the page contents
     page->getContents(&contents);
     if (contents->isStream()) {
-        obj1->initDict(contents->getStream()->getDict());
-        contents->getStream()->getDict()->incRef();
+        initDictFromDict (obj1, contents->streamGetDict());
+        contents->streamGetDict()->incRef();
         copyDict(&obj1);
         pdf_puts(">>\nstream\n");
         copyStream(contents->getStream()->getBaseStream());
@@ -810,14 +1018,16 @@ void write_epdf(void)
     else if (contents->isArray()) {
         pdfbeginstream();
         for (i = 0, l = contents->arrayGetLength(); i < l; ++i) {
-            contents->arrayGet(i, &obj1);
-            copyStream(obj1->getStream());
+        Object contentsobj;
+            copyStream((contents->arrayGet(i, &contentsobj))->getStream());
+        contentsobj.free();
         }
         pdfendstream();
     }
-    else
-        pdftex_fail("pdf inclusion: invalid page contents type <%s>", 
-                    contents->getTypeName());
+    else {// the contents are optional, but we need to include an empty stream
+        pdfbeginstream();
+        pdfendstream();
+    }
     // write out all indirect objects
     writeRefs();
     // write out all used encodings (and delete list)
@@ -827,16 +1037,19 @@ void write_epdf(void)
     pdf_doc->xref = xref;
 }
 
-// Called when an image has been written and it's resources in
-// image_tab are freed.  We cannot deallocate the PdfDocument yet, as
-// future pages of the same document may be embedded. As an
-// optimization, we do delete it if it's a one-page document.
-void epdf_delete()
+// Called when an image has been written and it's resources in image_tab are
+// freed and it's not referenced anymore.
+void 
+epdf_delete()
 {
     PdfDocument *pdf_doc = (PdfDocument *) epdf_doc;
     xref = pdf_doc->xref;
-    if (pdf_doc->doc->getCatalog()->getNumPages() == 1)
-	delete_document(pdf_doc);
+    if (pdf_doc->occurences < 0) {
+#ifdef DEBUG
+        fprintf(stderr, "\nDeleting %s\n", pdf_doc->file_name);
+#endif
+        delete_document(pdf_doc);
+    }
 }
 
 // Called when PDF embedding system is finalized.
@@ -845,15 +1058,12 @@ void epdf_check_mem()
 {
     if (isInit) {
         PdfDocument *p, *n;
-	for (p = pdfDocuments; p; p = n) {
-	    n = p->next;
-	    delete_document(p);
-	}
-	// see above for initParams
-	gfree(fontPath);
-	gfree(devFontMap);
-	// freeParams();
-        Object::memCheck(errFile);
-        gMemReport(errFile);
+    for (p = pdfDocuments; p; p = n) {
+        n = p->next;
+        delete_document(p);
+    }
+    // see above for globalParams
+    delete globalParams;
     }
 }
+// vi:ts=4:tw=79:expandtab:ai:
