@@ -1,3 +1,25 @@
+/*
+Copyright (c) 1996-2002 Han The Thanh, <thanh@pdftex.org>
+
+This file is part of pdfTeX.
+
+pdfTeX is free software; you can redistribute it and/or modify
+it under the terms of the GNU General Public License as published by
+the Free Software Foundation; either version 2 of the License, or
+(at your option) any later version.
+
+pdfTeX is distributed in the hope that it will be useful,
+but WITHOUT ANY WARRANTY; without even the implied warranty of
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+GNU General Public License for more details.
+
+You should have received a copy of the GNU General Public License
+along with pdfTeX; if not, write to the Free Software
+Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+$Id: //depot/Build/source.development/TeX/texk/web2c/pdftexdir/ttf2afm.c#3 $
+*/
+
 /*
 #include <kpathsea/c-auto.h>
 #include <kpathsea/c-fopen.h>
@@ -139,6 +161,7 @@ typedef struct {
 
 char *FontName = 0;
 char *FullName = 0;
+char *FamilyName = 0;
 char *Notice = 0;
 TTF_LONG ItalicAngle = 0;
 TTF_LONG IsFixedPitch = 0;
@@ -526,7 +549,7 @@ void read_font()
         k = get_ushort();
         l = get_ushort();
         if ((platform_id == 1 && encoding_id == 0) &&
-            (k == 0 || k == 4 || k == 6)) {
+            (k == 0 || k == 1 || k == 4 || k == 6)) {
             seek_off("name", j + get_ushort());
             for (p = buf; l-- > 0; p++)
                 *p = get_char();
@@ -534,10 +557,11 @@ void read_font()
             p = xstrdup(buf);
             switch (k) {
             case 0:  Notice = p; break;
+            case 1:  FamilyName = p; break;
             case 4:  FullName = p; break;
             case 6:  FontName = p; break;
             }
-            if (Notice != 0 && FullName != 0 && FontName != 0)
+            if (Notice != 0 && FamilyName != 0 && FullName != 0 && FontName != 0)
                 break;
         }
         i += 6*TTF_USHORT_SIZE;
@@ -625,6 +649,7 @@ void print_afm(char *date, char *fontname)
     fputs("\nStartFontMetrics 2.0\n", outfile);
     print_str(FontName);
     print_str(FullName);
+    print_str(FamilyName);
     print_str(Notice);
     fprintf(outfile, "ItalicAngle %i", (int)(ItalicAngle/0x10000));
     if (ItalicAngle%0x10000 > 0)
