@@ -1,3 +1,25 @@
+/*
+Copyright (c) 1996-2002 Han The Thanh, <thanh@pdftex.org>
+
+This file is part of pdfTeX.
+
+pdfTeX is free software; you can redistribute it and/or modify
+it under the terms of the GNU General Public License as published by
+the Free Software Foundation; either version 2 of the License, or
+(at your option) any later version.
+
+pdfTeX is distributed in the hope that it will be useful,
+but WITHOUT ANY WARRANTY; without even the implied warranty of
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+GNU General Public License for more details.
+
+You should have received a copy of the GNU General Public License
+along with pdfTeX; if not, write to the Free Software
+Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+$Id: //depot/Build/source.development/TeX/texk/web2c/pdftexdir/utils.c#7 $
+*/
+
 #include "ptexlib.h"
 #include "zlib.h"
 #include <kpathsea/c-vararg.h>
@@ -127,7 +149,7 @@ void pdf_printf(char *fmt,...)
     va_end(args);
 }
 
-strnumber maketexstring(char *s)
+strnumber maketexstring(const char *s)
 {
     int l;
     if (s == 0 || *s == 0)
@@ -180,6 +202,8 @@ void pdftex_fail(char *fmt,...)
     safe_print(print_buf);
     va_end(args);
     println();
+    safe_print(" ==> Fatal error occurred, the output PDF file is not finished!");
+    println();
     exit(-1);
 }
 
@@ -242,14 +266,6 @@ void setjobid(int year, int month, int day, int time, int pdftexversion, int pdf
 
     pdftexbanner = maketexstring(BANNER);
     
-    /* Initialize /Producer string */
-    sprintf(s,"pdfTeX-%d.%d%s-pdfcrypt", pdftexversion/100, pdftexversion%100, &pdftexrevision);
-    pdfcryptversion = maketexstring(s);
-
-    /* Initialize /CreationDate string */
-    sprintf(s,"D:%4d%02d%02d%02d%02d00", year, month, day, time/60, time%60);
-    pdfcryptdate = maketexstring(s);
-
     xfree(name_string);
     xfree(format_string);
     xfree(s);
