@@ -50,11 +50,13 @@ const_string ETEXHELP[] = {
     "                          output file",
     "-ipc-start               as -ipc, and also start the server at the other end",
 #endif /* IPC */
+    "-jobname=STRING          set the job name to STRING",
     "-kpathsea-debug=NUMBER   set path searching debugging flags according to",
     "                          the bits of NUMBER",
     "[-no]-mktex=FMT          disable/enable mktexFMT generation (FMT=tex/tfm)",
     "-mltex                   enable MLTeX extensions such as \\charsubdef",
     "-output-comment=STRING   use STRING for DVI file comment instead of date",
+    "-parse-first-line        parse of the first line of the input file",
     "-progname=STRING         set program (and fmt) name to STRING",
     "-recorder                enable filename recorder",
     "-shell-escape            enable \\write18{SHELL COMMAND}",
@@ -97,9 +99,11 @@ const_string MFHELP[] = {
     "                          true if the program name is `inimf'",
     "-interaction=STRING      set interaction mode (STRING=batchmode/nonstopmode/",
     "                          scrollmode/errorstopmode)",
+    "-jobname=STRING          set the job name to STRING",
     "-kpathsea-debug=NUMBER   set path searching debugging flags according to",
     "                          the bits of NUMBER",
     "[-no]-mktex=FMT          disable/enable mktexFMT generation (FMT=mf)",
+    "-parse-first-line        parse of the first line of the input file",
     "-progname=STRING         set program (and base) name to STRING",
     "-recorder                enable filename recorder",
     "-translate-file=TCXNAME  use the TCX file TCXNAME",
@@ -136,9 +140,11 @@ const_string MPHELP[] = {
     "                          true if the program name is `inimpost'",
     "-interaction=STRING      set interaction mode (STRING=batchmode/nonstopmode/",
     "                          scrollmode/errorstopmode)",
+    "-jobname=STRING          set the job name to STRING",
     "-kpathsea-debug=NUMBER   set path searching debugging flags according to",
     "                          the bits of NUMBER",
     "-mem=MEMNAME             use MEMNAME instead of program name or a %& line",
+    "-parse-first-line        parse of the first line of the input file",
     "-progname=STRING         set program (and mem) name to STRING",
     "-recorder                enable filename recorder",
     "-tex=TEXPROGRAM          use TEXPROGRAM for text labels.",
@@ -171,7 +177,6 @@ const_string OMEGAHELP[] = {
     "  If no arguments or options are specified, prompt for input.",
     "",
     "-file-line-error-style   print file:line:error style messages",
-    "-fmt=FMTNAME             use FMTNAME instead of program name or a %& line",
     "-ini                     be iniomega, for dumping formats; this is implicitly",
     "                          true if the program name is `iniomega'",
     "-interaction=STRING      set interaction mode (STRING=batchmode/nonstopmode/",
@@ -181,10 +186,13 @@ const_string OMEGAHELP[] = {
     "                          output file",
     "-ipc-start               as -ipc, and also start the server at the other end",
 #endif /* IPC */
+    "-jobname=STRING          set the job name to STRING",
     "-kpathsea-debug=NUMBER   set path searching debugging flags according to",
     "                          the bits of NUMBER",
     "[-no]-mktex=FMT          disable/enable mktexFMT generation (FMT=tex/tfm)",
+    "-oft=FMTNAME             use FMTNAME instead of program name or a %& line",
     "-output-comment=STRING   use STRING for DVI file comment instead of date",
+    "-parse-first-line        parse of the first line of the input file",
     "-progname=STRING         set program (and fmt) name to STRING",
     "-recorder                enable filename recorder (always on)",
     "-shell-escape            enable \\write18{SHELL COMMAND}",
@@ -229,10 +237,12 @@ const_string PDFTEXHELP[] = {
     "                          output file",
     "-ipc-start               as -ipc, and also start the server at the other end",
 #endif /* IPC */
+    "-jobname=STRING          set the job name to STRING",
     "-kpathsea-debug=NUMBER   set path searching debugging flags according to",
     "                          the bits of NUMBER",
     "[-no]-mktex=FMT          disable/enable mktexFMT generation (FMT=tex/tfm)",
     "-output-comment=STRING   use STRING for PDF file comment instead of date",
+    "-parse-first-line        parse of the first line of the input file",
     "-progname=STRING         set program (and fmt) name to STRING",
     "-recorder                enable filename recorder",
     "-shell-escape            enable \\write18{SHELL COMMAND}",
@@ -278,10 +288,12 @@ const_string PDFETEXHELP[] = {
     "                          output file",
     "-ipc-start               as -ipc, and also start the server at the other end",
 #endif /* IPC */
+    "-jobname=STRING          set the job name to STRING",
     "-kpathsea-debug=NUMBER   set path searching debugging flags according to",
     "                          the bits of NUMBER",
     "[-no]-mktex=FMT          disable/enable mktexFMT generation (FMT=tex/tfm)",
     "-output-comment=STRING   use STRING for PDF file comment instead of date",
+    "-parse-first-line        parse of the first line of the input file",
     "-progname=STRING         set program (and fmt) name to STRING",
     "-recorder                enable filename recorder",
     "-shell-escape            enable \\write18{SHELL COMMAND}",
@@ -327,11 +339,13 @@ const_string TEXHELP[] = {
     "                          output file",
     "-ipc-start               as -ipc, and also start the server at the other end",
 #endif /* IPC */
+    "-jobname=STRING          set the job name to STRING",
     "-kpathsea-debug=NUMBER   set path searching debugging flags according to",
     "                          the bits of NUMBER",
     "[-no]-mktex=FMT          disable/enable mktexFMT generation (FMT=tex/tfm)",
     "-mltex                   enable MLTeX extensions such as \\charsubdef",
     "-output-comment=STRING   use STRING for DVI file comment instead of date",
+    "-parse-first-line        parse of the first line of the input file",
     "-progname=STRING         set program (and fmt) name to STRING",
     "-recorder                enable filename recorder",
     "-shell-escape            enable \\write18{SHELL COMMAND}",
